package graphic;

import setup.*;
import server.*;
import server.event.*;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import java.awt.event.*;

/**
 * Memoryframe is the service-implementing JFrame that remembers the JFrames positioning,
 * and the size of the JFrame. It saves these data in the 'setup.conf' file. The frame also
 * implements a few common things for all frames. Such as hotkeys and closing for all frames.
 */

public abstract class MemoryFrame extends JFrame implements KeyListener, ConfObserver{
	
	private XMLTag xml;
	private String tagName;
	private static Vector v;
	protected static ResourceBundle res;

	/**
	 * Set-Construcor.
	 *
	 * @param   name  visual name of the frame.
	 * @param   tagName  The tagname, that the frame will have in 'setup.conf'.
	 */
	public MemoryFrame(String name, String tagName) {
		super(name);
		this.tagName = tagName;
		
		if (v == null) {
			v = new Vector();
		}
			
		v.add(this);
		
		addKeyListener(this);
		
		XMLTag tag = Conf.getRoot();
		XMLTag vinduer;

		try {
			vinduer = tag.getTagLike("Windows");
		}catch(NoSuchTagException nste) {
			vinduer = new XMLTag("Windows");
			tag.addTag(vinduer);	
		}
		
		try {
			xml = vinduer.getTagLike(tagName);
		}catch(NoSuchTagException nste) {
			xml = new XMLTag(tagName);
			vinduer.addTag(xml);	
		}
		
		if (res == null) {
			String tagname = "Memory";
			String optionname = "language";
			Locale locale = Locale.getDefault();
			XMLTag memory;
			
			try {
				memory = tag.getTagLike(tagname);
			}catch(NoSuchTagException nste) {
				memory = new XMLTag(tagname);
				tag.addTag(memory);
			}
			
			try {
				String option = memory.getOptionValue(optionname);
				locale = new Locale(option.substring(0,2),option.substring(3,5));
			}catch(NoSuchOptionException nsoe) {
				memory.addOption(optionname,locale.toString());
			}
			res = ResourceBundle.getBundle("graphic.resources.FTPResources",locale);
		}
		
		Conf.addObserver(this);
	}
	

	/**
	 * Set-Construcor.
	 *
	 * @param   name  the visual name of the frame, and the name of the tag.
	 */
	public MemoryFrame(String name) {
		this(name,name);
	}
	

	/**
	 * Closes all JFrames in the system. By calling this method,
	 * the system will close, and all positions are saved!
	 */
	public void closeAllWindows() {
		v.remove(this);
		try {
			((MemoryFrame)v.lastElement()).closeAllWindows();
		}catch(NoSuchElementException nse) {}
		dispose();
	}
	

	/**
	 * Sets the frame's state to Frame.Normal, and calls on to super-class.
	 *
	 * @param   b  visible?
	 */
	public void setVisible(boolean b) {
		super.setVisible(b);
		setState(Frame.NORMAL);
	}
	

	/**
	 * Overriding metod makes shure the positioning and size is saved before close.
	 */
	public void dispose() {
		save();
		super.dispose();
	}
	

	/**
	 * Saves positioning and size.
	 *
	 */
	public void save() {
		savePosition();
		saveSize();
	}
	

	/**
	 * Saves the positioning of the window to 'setup.conf'
	 *
	 */
	public void savePosition() {

		Point p = getLocation();
		int x = new Double(p.getX()).intValue();
		int y = new Double(p.getY()).intValue();
		
		try {
			xml.setOptionValue("pos-x",""+x);
			xml.setOptionValue("pos-y",""+y);
		}catch(NoSuchOptionException nsoe) {
			xml.addOption("pos-x",""+x);
			xml.addOption("pos-y",""+y);
		}
	}
	

	/**
	 * Saves the size of the window to 'setup.cof'.
	 *
	 */
	public void saveSize() {
		Dimension d = getSize();
		
		int x = getWidth();
		int y = getHeight();
		
		try {
			xml.setOptionValue("size-x",""+x);
			xml.setOptionValue("size-y",""+y);
		}catch(NoSuchOptionException nsoe) {
			xml.addOption("size-x",""+x);
			xml.addOption("size-y",""+y);
		}
	}
	

	/**
	 * Recursive method, that sets a KeyListener to all components in a Container, 
	 * and to their sub-components and so on...
	 *
	 * @param   c  The container, that contains all the components. Starting with ContentPane.
	 */
	private void addContainer(Container c) {
		Component[] all = c.getComponents();
		for (int i = 0 ; i < all.length ; i++) {
			all[i].addKeyListener(this);
			if (all[i] instanceof Container) {
				addContainer((Container)all[i]);
			}
		}
	}
	
	public void configurationFileReloaded() {
		initializeData();
	}
	
	protected void initializeData() {
	}
	

	/**
	 * Restores the Window to a certain position and size. 
	 * Also adds a container listener to all containers in the Window, so keyEvents is heard!
	 */
	public void restore() {
		addContainer(getContentPane());
		try {
			setSize(Integer.parseInt(xml.getOptionValue("size-x")), Integer.parseInt(xml.getOptionValue("size-y")));
			setLocation(Integer.parseInt(xml.getOptionValue("pos-x")), Integer.parseInt(xml.getOptionValue("pos-y")));
		}catch(NoSuchOptionException nsoe) {
			centerWindow();
		}catch(NumberFormatException nfe)  {
			centerWindow();
		}
	}
	

	/**
	 * Centers the window on the screen.
	 *
	 */
	private void centerWindow() {
		Dimension d = getToolkit().getScreenSize();
		
		setLocation(d.width/2 - getWidth()/2, d.height/2 - getHeight()/2);
	}	

	/**
	 * Finds CTRL+ALT+E, and opens the ErrorConsole!
	 *
	 * @param   e  KeyEvent
	 */
	public void keyPressed(KeyEvent e) {
		if (e.getKeyCode() == e.VK_E && e.getModifiers() == 10) {
			ErrorConsole.instanceOf().setVisible(true);
		}
	}
	

	/**
	 * I had to implement it! ;o) It does nothing!
	 *
	 * @param   e  KeyEvent
	 */
  public void keyReleased(KeyEvent e) {
  }
	

	/**
	 * I had to implement it! ;o) It does nothing!
	 *
	 * @param   e  KeyEvent
	 */
  public void keyTyped(KeyEvent e) {
  }
	
	public void setMyLocale(Locale locale) {
		res	= ResourceBundle.getBundle("graphic.resources.FTPResources",locale);
		for(int i = 0 ; i < v.size() ;i++)
			((MemoryFrame)v.get(i)).setText();
	}
	
	public abstract void setText();

	
}