package server;

import java.util.*;
import java.io.*;

import server.services.*;
import server.event.*;

/**
 * JQ-Data FTP-Server must be started though this class. When started all clients can connect to the server
 * on the running port (typically 21). All standard setups are in the setup.xml, that are generated the first
 * time the server runs. <br> If you want to change the port to fx. 3000, you can do it by doing:
 * <br><code>getConfiguration().getRoot().setQuickOptionValue("Server.port","3000");</code><br>
 * The one that makes an instance of this class will often be <a href='event/ServerObserver.html'>ServerObserver</a>,
 * because it thereby gets all newcommers on the server send to it though ServerObservers interface!
 * The Object <a href='UserSession.html'>UserSession</a> represents a client's session.
 *
 */
public class Server {
	private Dock d;
	private HashSet serv;
	private static Server server;
	/** The Servers version. Used by the two commands 'Welcome' og 'Help'*/
	public static String version;

	/**
	 * Makes a new instance of a Server. 
	 *
	 * @param   version  a name that is used in GUI, help and in commandline (java -jar JQData-FTP.jar -version)
	 */
	private Server(String version) {
		this.version = version;
		serv = new HashSet();
	
		Conf.getInstance();
    d = new Dock(serv);
	}
	

	/**
	 * Creates a singleton server
	 *
	 * @param   version  a name that is used in GUI, help and in commandline (java -jar JQData-FTP.jar -version). Only the first one to call this method gets the version name through,
	 * @return     instance
	 */
	public static Server instanceOf(String version) {
		if (server == null)
			server = new Server(version);
		return server;
	}
	

	/**
	 * Creates a singleton server, with a version name of ''.
	 *
	 * @return     instance
	 */
	public static Server instanceOf() {
		return instanceOf("");
	}


	/**
	 * Tries to start the FTP-Server as a thread. The server calls back through the interface 'ServerObserver', 
	 * and states whether it was successfull or not.
	 */
	public void start() {
		Chain.sortChains();
		d.start();
	}
	

	/**
	 * Stops the FTP-Server.
	 *
	 */
	public boolean stop() {
		Chain.saveChains();
		UserSession[] us = getOnlineUsers();
		for(int i = 0 ; i < us.length ; i++) {
			us[i].abortTransmission();
			us[i].disconnectClient(UserSessionObserver.USER_KICKED_DUE_TO_SERVER_STOP);
		}
		Conf.save();
		return d.stop();
	}
		

	/**
	 * Adds an observer.
	 *
	 * @param   serverObserver  observer
	 */
	public void addServerObserver(ServerObserver serverObserver) {
		serv.add(serverObserver);
	}
	

	/**
	 * Removes an observer
	 *
	 * @param   serverObserver  observer
	 */
	public void removeServerObserver(ServerObserver serverObserver) {
		serv.remove(serverObserver);
	}
	

	/**
	 * Returns the configuration class. Can also be obtained by Conf.getInstance();
	 *
	 * @return     The configuration classen.
	 */
	public Conf getConfiguration() {
		return Conf.getInstance();
	}
	

	/**
	 * Wether the server is running or not.
	 *
	 * @return     true/false
	 */
	public boolean isRunning() {
		return d.isRunning();
	}
	

	/**
	 * The port the server will run on.
	 *
	 * @return     portnummer
	 */
	public int getServerPort() {
		return d.getServerPort();
	}
	
	/**
	 * Sets the server to auto-save. This will save setup.conf every time a client goes offline
	 * This is fx. necessary when running without graphics, as <pre><Hukommelse>sessioner 1</Hukommelse></pre>
	 * must be saved before the server are closed...<br><br>
	 *
	 * This option is by default set to OFF!
	 *
	 * @param   on  
	 */
	public void setAutoSave(boolean on) {
		d.setAutoSave(on);
	}
	

	/**
	 * Checks whether autoSave is on or off.
	 *
	 * @return     true/false
	 */
	public boolean doesAutoSave() {
		return d.doesAutoSave();
	}
	

	/**
	 * Returns an array containing all online clients.
	 *
	 * @return     all online clients.
	 */
	public UserSession[] getOnlineUsers() {
		HashSet users = d.getUsers();
		UserSession[] arr = new UserSession[users.size()];
		
		Iterator it = users.iterator();
		
		for (int i = 0; it.hasNext() ; i++) {
			arr[i] = ((Probe)it.next()).ses.us;
		}
		
		return arr;
	}
	

	/**
	 * Gets all logfiles on the FTP-Server.
	 *
	 * @return     LogFileReader[]
	 */
	public LogFileReader[] getLogFiles() {
		String[] files = new File("log").list();
		if (files==null)
			files = new String[0];
		int leng = 0;
		
		//finds the size for the array
		for (int i = 0 ; i < files.length ; i++)
			if (files[i].endsWith(".log"))
				leng++;
			
		LogFileReader[] arr = new LogFileReader[leng];
		
	  //puts all into the array
		leng=0;
		for (int i = 0 ; i < files.length ; i++)
			if (files[i].endsWith(".log"))
				arr[leng++] = new LogFileReader(files[i].substring(0,files[i].length()-4));
		
		return arr;
	}
	

	/**
	 * Deletes a group of LogFileSession from the logfiles. The array of LogFileSessions
	 * can consist of objects from severel different users. If there are errors in one or
	 * some of the logfiles, the method returns false, but this is not a guarrantee, that
	 * some logfiles were not deleted!
	 *
	 * @param   ses  an array consisting of LogFileSession objects to be deleted
	 * @return     true if the logs were successfully deleted!
	 */
	public boolean deleteLogFileSessions(LogFileSession[] ses) {
		//Here all sessions are devided between the respektive users.
		//They are put into vectors, that are inside a HashMap.
		HashMap map = new HashMap();
		int lng = ses.length;
		for (int i = 0 ; i < lng ; i++) {
			Vector v = (Vector) map.get(ses[i].getLogin());
			if (v == null)
				v = new Vector();
			v.add(ses[i]);
			map.put(ses[i].getLogin(),v);
		}
		
		//Makes LogFileReaders that are instructed to delete some of their sessions.
		Iterator i = map.values().iterator();
		LogFileReader reader;
		Vector v;
		boolean ok=true;
		while (i.hasNext()) {
				v = (Vector) i.next();
				reader = new LogFileReader(((LogFileSession)v.firstElement()).getLogin());
				if (!reader.deleteSessions(copyToArray(v)))
					ok=false;
		}
		return ok;
	}
	
	//Copies all the objects from the vector into a LogFileSession array.
	private LogFileSession[] copyToArray(Vector v) {
		LogFileSession[] arr = new LogFileSession[v.size()];
		for (int i = 0 ; i < arr.length; i++) {
			arr[i] = (LogFileSession) v.get(i);
		}
		return arr;
	}
}