package server;

import server.event.*;
import server.services.*;

import java.util.*;
import java.io.*;

/**
 * Keeps an eye on the clients and kicks/bans them if they are too passive, 
 * or uses a known pattern of commands that should be banned!
 */

public final class Survailance implements ServerObserver{

	private boolean started = false;
	private static int secondsToTimeout = 300;
	private static Vector kickPatterns = new Vector();
	private static HashSet controllers = new HashSet();
	private static Survailance sur;
	public final static int KICK_ON_FOUND_PATTERN=1;
	public final static int BAN_ON_FOUND_PATTERN=2;
	/** Default = BAN_ON_FOUND_PATTERN*/
	public static int ACTION_ON_FOUND_PATTERN=2;
	private HashMap clients = new HashMap();
	private Timer timer;
	private File patternFile = new File("ban-patterns.conf");
	private LogFileWriter fw = new LogFileWriter();
	private LogFileReader fr = new LogFileReader(patternFile,"Ban-Patterns");

	private Survailance() {
		Server.instanceOf().addServerObserver(this);
		readloadPatterns();	
		if (Conf.IsIntelligenceActive())
			start();
	}
	

	/**
	 * Constructor of Singleton class.
	 *
	 * @return     the instance
	 */
	public static Survailance instanceOf() {
		if (sur == null)
			sur = new Survailance();
		return sur;	
	}
	

	/**
	 * Sees if the LogFileSession already exists in the banpatterns.
	 *
	 * @param   logFileSession pattern to check if exists 
	 * @return     if the pattern is already added, the sessionnumber that holds the pattern are returned. -1 otherwise.
	 */
	public int containsPattern(LogFileSession logFileSession) {		
		if (!patternFile.exists())
			return -1;
			
		int num[] = fr.getSessionNumbers();
		LogFileSession ses;
		Vector nc = logFileSession.getConversation();
		Vector oc;
		for (int i = 0 ; i < num.length ; i++) {
			ses = fr.getSession(num[i]);
			oc = ses.getConversation();
			if (nc.size() == oc.size()) {
				int size = oc.size()-1;
				boolean ok=true;
				for (int j = 2; j < size ; j++) {
					if (!oc.get(j).toString().equals(nc.get(j).toString())) {
						ok = false;
						break;
					}
				if (ok)
					return ses.getSessionNr();	
				}	
			}
		}	
		return -1;
	}
	
	

	/**
	 * Gets the logFileReader reading the 'ban-patterns.conf' file.
	 *
	 * @return     file containing all ban-patterns
	 */
	public LogFileReader getLogFileReader() {
		return fr;	
	}
	

	/**
	 * Sets the number of seconds a client must live before he is kicked if he is too passive.
	 * One thing should be noticed though! Only ONE timer keeps track of all clients.
	 * This means that even if this method is set to 60 seconds, a new client cold be kicked 
	 * after 10 seconds, because the timer has reached 50 seconds when he connects...
	 * NO clients can be kicked within their first 10 seconds online though...
	 *
	 * @param   seconds  Seconds before autokick if passive
	 */
	public void setSecondsToTimeout(int seconds) {
		if (secondsToTimeout != seconds) {
			secondsToTimeout = seconds;
			boolean before = started;
			stop();
			if (before)
				start();
		}	
	}
	

	/**
	 * When a client connects this method is automatically called. 
	 * A client cannot be added though here twice (uses HashMap for storage)
	 *
	 * @param   userSession  the new client
	 */
	public final synchronized void clientConnected(UserSession userSession) {
		Client client = new Client(userSession);
		clients.put(userSession,client);
	}
	

	/**
	 * When a client disconnects this method is automatically called.
	 *
	 * @param   userSession  the disconnecting client
	 */
	public final synchronized void clientDisconnected(UserSession userSession) {
		Client client = (Client) clients.remove(userSession);
	}

	public void serverStarted(boolean startedOk) {
	}

	public void serverStoped() {
	}
	

	/**
	 * Adds a LogFileSession as a pattern, that clients will be kicked if they use!
	 *
	 * @param   ses  pattern
	 * @return     if the pattern is already added, the sessionnumber that holds the pattern are returned. -1 otherwise.
	 */
	public int addKickPattern(LogFileSession ses) {
		int sesnr = containsPattern(ses);
		if ( sesnr == -1 ) {
			fw.write(ses,patternFile);
			addFormattedPattern(ses);
		}
		return sesnr;

	}
	

	/**
	 * Delets a kickpattern from the survailance
	 *
	 * @param   ses  pattern
	 * @return     true if found
	 */
	public boolean deleteKickPattern(LogFileSession ses) {
		LogFileSession[] arr = {ses};
		boolean ok = fr.deleteSessions(arr);
		readloadPatterns();
		return ok;
	}
	

	/**
	 * Reloads all the patterns from the ban-patterns.conf file
	 *
	 */
	public void readloadPatterns() {
		kickPatterns.clear();
		if (patternFile.exists()) {
			int num[] = fr.getSessionNumbers();
			for (int i = 0 ; i < num.length ; i++)
				addFormattedPattern(fr.getSession(num[i]));
		}
	}
	

	/**
	 * Adds all LogFileSession to patterns kickPatterns Vector
	 *
	 * @param   ses  pattern to add
	 */
	private void addFormattedPattern(LogFileSession ses) {
		Vector conv = ses.getConversation();	  
		Vector tmp = new Vector();
		int size = conv.size();
		for (int i = 2 ; i < size ; i++) { // i is 2 because username and password is ignored when finding patterns
			if (conv.get(i).toString().startsWith("C"))
				tmp.add(conv.get(i).toString().substring(2));
		}
		
		size = tmp.size();
		if (size > 2) { //To avoid negative arrays...
			String[] str = new String[size-2];
			for (int i = 1 ; i < size-1 ; i++) {
				str[i-1] = tmp.get(i).toString();
			}
			kickPatterns.add(str);
		}
	}
	

	/**
	 * Gets all kickpatterns in a new Vector.
	 *
	 * @return     all petterns in a cloned Vector.
	 */
	public Vector getKickPatterns() {
		return (Vector) kickPatterns.clone();
	}
	

	/**
	 * Adds a SurvailanceController. 
	 *
	 * @param   survailanceController  
	 */
	public void addSurvailanceController(SurvailanceController survailanceController) {
		controllers.add(survailanceController);
	}
	

	/**
	 * Removes a SurvailanceController
	 *
	 * @param   survailanceController  
	 */
	public void removeSurvailanceController(SurvailanceController survailanceController) {
		controllers.remove(survailanceController);
	}
	

	/**
	 * Calls all SurvailanceControllers to see if just ONE of them will let this session continue.
	 * If this is the case, the session will get his time prolonged on the server. If not,
	 * the client will be kicked after XX time if another command has not made a SurvailanceControllers
	 * return true...
	 * No more SurvailanceControllers will be asked if one returns true!
	 *
	 * @param   command  the clients command
	 * @param   us  the Usersession/client that said the command
	 * @return     true if the client may continue.
	 */
	private boolean letSessionContinue(String command, UserSession us) {
		Iterator it = controllers.iterator();
		boolean cont = false;
		while(it.hasNext())
			if (((SurvailanceController)it.next()).letSessionContinue(command,us))
				cont = true;
		return cont;	
	}
	

	/**
	 * Calls all SurvailanceControllers to see if one of them want the client disconnected.
	 * If just ONE returns true, the client will be disconnected in a heartbeat...
	 * No more SurvailanceControllers will be asked if one returns true!
	 *
	 * @param   command  the clients command
	 * @param   us  the Usersession/client that said the command
	 * @return     true if kick
	 */
	private boolean disconnectSession(String command, UserSession us) {
		Iterator it = controllers.iterator();
		while(it.hasNext())
			if (((SurvailanceController)it.next()).disconnectSession(command,us))
				return true;
		return false;	
	}
	

	/**
	 * Starts the survailance of clients.
	 *
	 */
	public synchronized void start() {
		started = true;
		if (timer != null)
			timer.interrupt();
		timer = new Timer();
		timer.start();
	}
	

	/**
	 * Stops the survailance of clients.
	 *
	 */
	public synchronized void stop() {
		started = false;
		if ( timer != null )
			timer.interrupt();
	}
	

	/**
	 * Is Survailance turned on
	 *
	 * @return     
	 */
	public boolean isStarted() {
		return started;
	}
	
	/**
	 * Keeps an eye on a client. Sees if he needs to be kicked or banned ;o)
	 */
	class Client extends UserSessionObserverAdapter{
		private UserSession us;
		private boolean kick = true;
		private Vector matches = new Vector(); //  contains Object[] = { place in kickPatterns, place in the pattern it self }
		private Date startup = new Date();
		
		public Client(UserSession us) {
		  this.us = us;
			us.addUserSessionObserver(this);
		}
		
		public void commandUpdate(String command) {
			if (letSessionContinue(command, us))
				kick = false;
			if (disconnectSession(command,us)) {
				kick = true;
				timeIsUp(true);
			}
			if (match(command) && started) {
				switch (ACTION_ON_FOUND_PATTERN) {
					case BAN_ON_FOUND_PATTERN:
						Conf.banAddress(us.getClientInetAddress());
						us.disconnectClient(UserSessionObserver.USER_AUTO_BANNED);
						break;
					case KICK_ON_FOUND_PATTERN:
						us.disconnectClient(UserSessionObserver.USER_KICKED);
				}
			}
		}
		
		private boolean match(String command) {
			String[] pattern;
			Object[] looking;
			int place;
			for (int i = matches.size()-1 ; i >= 0 ; i--) {
				looking = (Object[]) matches.get(i);
				pattern = (String[])looking[0];
				place = ((Integer)looking[1]).intValue()+1;
				if (pattern[place].equalsIgnoreCase(command)) {
					if (pattern.length == place+1) {
						matches.remove(looking);
						return true;
					}else {
						looking[1] = new Integer(place);
					}
				}else
					matches.remove(looking);
			}

			for (int i = kickPatterns.size()-1 ; i >= 0 ; i--) {
				pattern = (String[]) kickPatterns.get(i);
				if (pattern[0].equalsIgnoreCase(command))
					matches.add(new Object[] {pattern,new Integer(0)});
			}
			return false;
		}
		
		public void timeIsUp(boolean force) {
			//check (Note that NO clients can be kicked within the first 10 seconds)
			if ((kick && started && (!(us.sendingFile() && us.getSpeedInBytesPrSek() > 0)) && (new Date().getTime() - startup.getTime()>10000)) || force) {
				us.disconnectClient(UserSessionObserver.USER_TIMEOUT);
			}
			if (!us.sendingFile())
				kick=true;
		}
	}
	
	/**
	 * This timer counts down, and starts the validating of all clients online, 
	 * to kick anyone who is too passive.
	 */ 
	class Timer extends Thread {
		
		public void run() {
			while (true) {
				try {
					sleep(secondsToTimeout*1000);
					Iterator it = clients.keySet().iterator();
					Client c;
					while (it.hasNext()) {
						c = (Client)clients.get(it.next());
						c.timeIsUp(false);
					}
				}catch(ConcurrentModificationException e) {
					//If this happens, we just stop iterating, and start again next time the timer starts the itaration.
				}catch(InterruptedException ie) {
					break;
				}
			}
		}
	}
}