package server;

import java.util.*;
import java.net.*;
import java.io.*;
import java.text.*;

import setup.*;
import server.services.*;
import server.event.*;

/**
 * The class represents a session from a client. It is automatically instantiated by the server, when the client connects,
 * and are send though the <a href='event/ServerObserver.html'>ServerObserver</a> interface to the observers.
 *
 */
public abstract class UserSession implements Session{

	private server.services.Session ses;
	private Command com;
	/**All the UserSessionObservers on this UserSession*/
	protected HashSet obs = new HashSet();
	private Date date;
	/**@see server.event.UserSessionObserver server.event.UserSessionObserver*/
	protected int state=-1; //online,kicked,banned and so on...
	

	/**
	 * ONLY a Server (Probe) can and may 'make instances' of this class!
	 */
	protected UserSession() {
		date = new Date();
	}
	

	/**
	 * Sets the classes references. Made, because it cannot be set in the constructors,
	 * because the objects is instantiated in a certain order.
	 *
	 * @param   ses  Session
	 * @param   com  Command
	 */
	protected void set(server.services.Session ses, Command com) {
		this.ses = ses;
		this.com = com;
	}


	/**
	 * Adds an observer
	 *
	 * @param   userSessionObserver  observer
	 */
	public void addUserSessionObserver(UserSessionObserver userSessionObserver) {
		obs.add(userSessionObserver);
	}
	

	/**
	 * Removes an observer
	 *
	 * @param   userSessionObserver  
	 */
	public void removeUserSessionObserver(UserSessionObserver userSessionObserver) {
		obs.remove(userSessionObserver);
	}
	

	/**
	 * Returns the clients InetAddresse
	 *
	 * @return     InetAddress
	 */
	public InetAddress getClientInetAddress() {
		return ses.clientCommandSocket.getInetAddress();
	}

	public String getClientIP() {
		return getClientInetAddress().getHostAddress();
	}


	/**
	 * Returns the clients own XMLTag. This can be used to find information about the client.
	 * For this purpose the Conf can be used as a service calss for some operations.
	 *
	 * @return     This clients own XMLTag.
	 */
	public XMLTag getClientTag() {
		return ses.xml;
	}
	
	
	/**
	 * Whether the current file is upload to the client or not. If nothing is currently transmitted, false are returned.
	 *
	 * @return     true if the server is currently uploading. (the client is downloading)
	 */
	 public boolean isFileUpload() {
		if (ses.fileHandler == null)
			return false;
		return (ses.fileHandler.MODE_SEND_FILE == ses.fileHandler.getSendMode());
	}
	

	/**
	 * Whether the current file is downloading from the client or not. If nothing is currently transmitted, false are returned.
	 *
	 * @return     true if the server is currently downloading. (the client is uploading)
	 */
	public boolean isFileDownload() {
		if (ses.fileHandler == null)
			return false;
		return (ses.fileHandler.MODE_RECIVE_FILE == ses.fileHandler.getSendMode());
	}
	

	/**
	 * The number for bytes per second, that the current file are transmitted with.
	 *
	 * @return     bytes per second. -1 if nothing is beeing transmitted.
	 */
	public int getSpeedInBytesPrSek() {
		if (ses.fileHandler == null)
			return -1;
		return ses.fileHandler.getSpeedInBytesPrSek();
	}
	

	/**
	 * The number of bytes, that the server may transfer to this client per second.
	 *
	 * @param   bytes  bytes per second
	 */
	public void setSpeedInBytesPrSek(int bytes) {
		if (ses.fileHandler != null)
			ses.fileHandler.setSpeedInBytesPrSek(bytes);
	}
	

	/**
	 * The number of bytes, that thas been transfered in the actuel filetransfer.
	 *
	 * @return     number of bytes. -1 if nothing is beeing transmitted.
	 */
	public int getSendBytesInFile() {
		if (ses.fileHandler == null)
			return -1;
		return ses.fileHandler.getSendBytes();
	}
	

	/**
	 * The total number of send bytes to this client. Directorylists are NOT included!
	 *
	 * @return     antal bytes
	 */
	public int getSendByteTotal() {
		//int[] arr =  {recived,sended,recivedBytes,sendedBytes};
		int[] arr = com.getStatArray();
		return arr[3];
	}
	

	/**
	 * The total number of recived bytes from this client.
	 *
	 * @return     antal bytes
	 */
	public int getRecivedByteTotal() {
		int[] arr = com.getStatArray();
		return arr[2];
	}
	

	/**
	 * The total number of send files to this client
	 *
	 * @return     number of send files
	 */
	public int getNumberOfSendFiles() {
		int[] arr = com.getStatArray();
		return arr[1];
	}
	

	/**
	 * The total number of files that has been recived from this client.
	 *
	 * @return     number of recived files.
	 */
	public int getNumberOfRecivedFiles() {
		int[] arr = com.getStatArray();
		return arr[0];
	}
	

	/**
	 * Aborts this clients current filetransmission.
	 *
	 */
	public void abortTransmission() {
		com.abortTransmission();
	}
	

	/**
	 * Is there an upload/download/directorytransfer going on?
	 *
	 * @return     true true if the server is currently transmitting
	 */
	public boolean sendingFile() {
		return com.DataSocketInUse();
	}
	

	/**
	 * This usersessions sessionnumber
	 *
	 * @return     nr
	 */
	public int getSessionNr() {
		return ses.sessionNr;
	}
	

	/**
	 * Kicks the client from the server, and cancels the filetransfer if any.
	 */
	public void disconnectClient(int reasonCode) {
		com.abortTransmission();
		ses.disconnectClient(reasonCode);
	}
	

	/**
	 * Writes a line to the client over the command-connection. It is often a good idea
	 * to start any line by writing [200 ] (without the []), as this is undestood as a comment
	 * by the client.
	 *
	 * @param   line  The text to send to the client!
	 */
	public void tellClient(String line) {
		com.run(line);
	}
	

	/**
	 * The login name of the client
	 *
	 * @return     login
	 */
	public String getLogin() {
		return ses.getLogin();
	}
	

	/**
	 * The time the client logged onto the server
	 *
	 * @return     time
	 */
	public Date getLogonTimeAsDate() {
		return date;
	}
	
	public String getLogonTime() {
		return new SimpleDateFormat("'D.' dd.MM.yyyy ' kl.' HH:mm:ss").format(date);
	}
	
	//Is not implemented. The interface is the reason it is here!
	public Vector getConversation() {
		return null;
	}
	

	/**
	 * Is the Client loged in?
	 *
	 * @return     true if yes
	 */
	public boolean isLogedIn() {
		return ses.logedIn;
	}

	/**
	 * States this client can bee in (online, kicked, banned ect.)
	 *
	 * @see server.event.UserSessionObserver server.event.UserSessionObserver
	 * @return
	 */
	public int getUserState() {
		return state;
	}

}