package server.commands;

import server.Conf;
import server.services.*;
import java.io.*;
import java.util.*;


/**
 * This is the superclass of ANY command! It formes the Chain-of-Responsibillity. 
 * The class <b>MUST</b> have the name of the FTP - command in order to work!
 * Fx. is the command cwd handled by the class Cwd.class.
 */
public abstract class AbstractCommand implements Comparable, Serializable{ 
  
  private AbstractCommand next,last=this;
  protected Command command;
	protected Session ses;
	protected String name;
	protected String originalOrder;
	private static HashMap counters = new HashMap();
	private Num num;
	
	/**
	 * This method holds the implementation for the command. 
	 * It is called with the result of a removeFirst(String order) call.
	 *
	 * @param   order the order, except the first word.
	 */
  protected abstract void carryOut(String order);
	
	public AbstractCommand() {
		name = getClass().getName();
		name = name.substring(name.lastIndexOf('.')+1, name.length()).toLowerCase();
		
		num = (Num) counters.get(name);
		if (num == null) {
			num = new Num();
			try {
				num.value = Conf.getCommandUsage(name);
			}catch(Exception e) {
				num.value = countUp();
			}
			counters.put(name,num);
		}
	}
	
	public final void save() {
		Conf.setCommandUsage(name,num.value);
		if (next != null)
			next.save();
	}
	
	
	public final AbstractCommand clone(Session ses, Command com) {
		AbstractCommand c=null;
		try {
			c = (AbstractCommand) getClass().newInstance();
			c.build(ses, com);
			if (next != null)
				c.next = next.clone(ses,com);
		}catch(Exception e) {
			e.printStackTrace();
		}
		return c;
	}
	
	/**
	 * Sends the Clients order to the Object. If AbstractCommand cannot handle the event, it
	 * will send it on to the next AbstractCommand in the Chain.<br>
	 * If the AbstractCommand can handle the event, the first word of the command is removed, 
	 * and the remainder is send on to the abstract method carryOut(String order).
	 *
	 * @param   originalOrder  client order (fx. cwd)
	 */
	public final void action(String originalOrder){
		this.originalOrder = originalOrder;
		if (originalOrder.toLowerCase().startsWith(name) || next==null) {
      carryOut(removeFirst(originalOrder));
			num.value = countUp();
		}else
      next.action(originalOrder);
  }
	
	protected int countUp() throws NullPointerException{
		return num.value-1;
	}
	
	public final AbstractCommand sort() {
		AbstractCommand n=this;
		TreeSet t = new TreeSet();
		while (n != null) {
			t.add(n);
			n=n.next;
		}
		AbstractCommand a=null,b;
		Iterator i = t.iterator();
		if (i.hasNext()) {
			a = (AbstractCommand) i.next();
		}
		while (i.hasNext()) {
			b = (AbstractCommand) i.next();
			a.next = b;
			a=b;
		}
		return (AbstractCommand) t.first();
	}
	
	public boolean equals(Object o) {
		return o.getClass().getName().equals(getClass().getName());
	}
	
	
	public int compareTo(Object o1) {
		Num a = (Num) counters.get(name);
		Num b = (Num) counters.get(((AbstractCommand) o1).name);
		if (a == null)
			a = new Num();
		if (b == null)
			b = new Num();
		if (a.value < b.value)
			return -1;
		if (a.value == b.value)
			return -1; //We have to do this, because the comparrisment is used to sort out which object goes in the TreeSet and which ones dosn't.
		else
			return 1;	
	}

  /**
	 * Empty method that can be overridden, and is called at instantiation time.
	 */	 
  protected void start() {
  }
		
	/**
	 * Is used to build up the chain.
	 *
	 * @param   next  the next object in the chain.
	 * @param   ses  the session that this chain represents
	 * @param   command  the command object, that is used to communicate with the client.
	 */
  public final void build(Session ses, Command command){
  	this.ses = ses;
  	this.command = command;
		start(); //must be after ses and command is set! (See server.commands.Port)
  }
	

	/**
	 * Adds a command as the second last commmand. 
	 *
	 * @param   command  the new AbstractCommand to add to the chain
	 */
	public final void addCommandLast(AbstractCommand command) {
		last.next=command;
		command.build(ses,this.command);
		last = command;
	}
    

	/**
	 * Removes the first word of the order.
	 *
	 * @param   originalOrder   fx. cwd
	 * @return     the remaining order.
	 */
  private String removeFirst(String originalOrder){
    int space = originalOrder.indexOf(' ');
		if (space != -1)
      return originalOrder.substring(space+1,originalOrder.length());
    else
      return "";
  }
  

	/**
	 * Makes a file of an order.
	 *
	 * @param   order  a filename (example.txt)
	 * @return     a File Object with the absolutePath to the file.
	 */
	protected File makeFileOfOrder(String order){
	  if (order.length()==0)
			return new File("");
			
			order = order.replace('/','\\');
			int pos = order.lastIndexOf(System.getProperty("file.separator").charAt(0));
		  if (pos != -1) {
				ses.fileSystem.cwd(order.substring(0,pos));
				order = order.substring(pos+1);
		  }

		String currentDir=ses.fileSystem.curDir.getAbsolutePath();
		String dest;
    char c = System.getProperty("file.separator").charAt(0);
    
	  if( order.charAt(0) == c )
    	dest = removeFirst(order).substring(1,order.length());
    else if (currentDir.charAt((int)currentDir.length()-1) == c)
      dest = currentDir + order;
		else
      dest = ""+ currentDir + c + order;
    
    return new File(dest);
	}
	

	/**
	 * Takes a file, and prints it in the correct way (Windows vs Unix)
	 *
	 * @param   f  File
	 * @return     Output
	 */
	protected String fileToSystem(File f) {
		char c = System.getProperty("file.separator").charAt(0);	
		if (System.getProperty("os.name").startsWith("Windows")) {
			if (f.getAbsolutePath().lastIndexOf(c) == f.getAbsolutePath().length()-1)
			  return f.getAbsolutePath(); //Windows syntax for roden (f.eks. C:\)
			else
				return f.getAbsolutePath() + c; //Windows syntax
		}else
			return c + f.getAbsolutePath() + c; //Unix syntax
	
	}
  

	/**
	 * Prints a comma seperated list of the the names in the Chain-of-responsibillity,
	 * from this AbstactCommand and forward.
	 *
	 * @return     String...
	 */
  public String toString(){
		return getClass().getName() + " " + num.value + ", " + next.toString();
  }
	
	class Num {
		public int value = 1;
		public String toString() {
			return ""+value;
		}
	}
}