package setup;

import java.util.*;
/**
 * The class represents a Tag as fx. &ltTAG&gt. The whole idea, is to make a dynamical XML-structure.
 * Typically the class is made through an XMLParser, by instantiating the XMLParser, and using its "main-tag" <br><br>
 *
 * <code>
 * XMLParser parser = new XMLParser(new File("setup.conf"));<br>
 * XMLTag xml = parser.XML;
 *<br><br>
 *</code>Some of the classes documentation builds on the following example.<code><br><br>
 * 
 *#Serversetup<br>
 *&ltServer&gt<br>
 *&nbsp;&nbsp;Port 21<br>
 *&lt/Server&gt<br>
 *<br>
 *#Users on the server!<br>
 *&ltUsers&gt<br>
 *&nbsp;&nbsp;&ltUser&gt<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;name Kenneth Vindum<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;login lkv<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;pass pkv<br>
 *&nbsp;&nbsp;&lt/User&gt<br>
 *&nbsp;&nbsp;&ltUser&gt<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;name John Doe<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;login lsr<br>
 *&nbsp;&nbsp;&nbsp;&nbsp;pass psr<br>
 *&nbsp;&nbsp;&lt/User&gt<br>
 *&lt/Users&gt<br><br>
 *
 *</code>
 *
 */
public class XMLTag{
  /** The name of the tag */
  public String tagName;
  private boolean finished=false;
  private boolean okToAdd = true;

  private Vector comments = new Vector();
  private Vector options = new Vector();
  private Vector tags = new Vector();
  
  private static Vector mem = new Vector();
	private XMLTag parent=null;

  /**
   * Set-constructor.
   *
   * @param   tagName  The name of the tag
   */
  public XMLTag(String tagName){
    this.tagName = tagName;
  }
  

  /**
   * Adds an option to a Tag. Fx. [FTPServerport 21], where optionName = FTPServerport, og optionValue=21
   *
   * @param   optionName  The options name
   * @param   optionValue  The options value
   */
  public void addOption(String optionName, String optionValue){
    if (okToAdd){
      String[] op = {optionName, optionValue};
      options.add(op);
    }else{
      XMLTag tag = (XMLTag) tags.lastElement();
      tag.addOption(optionName,optionValue);
    }
  }
	

	/**
	 * Deletes an option from the Tag.
	 *
	 * @param   optionName  The name of the option that should be deleted!
	 */
	public void removeOption(String optionName) throws NoSuchOptionException{
		String[] op;
		for (int i = 0 ; i < options.size(); i++) {
			op = (String[]) options.get(i);
			if (op[0].equals(optionName)) {
				options.remove(op);
				return;
			}
		}
		throw new NoSuchOptionException("Option ["+optionName+"] findes ikke i <"+tagName+">!");
	}
  

  /**
   * Adds a comment to the tag.
   * The method adds "#" in front of the tag, if it is not allready there.
   *
   * @param   comment  The comment
   */
  public void addComment(String comment){
    if (!comment.startsWith("#") && (comment.trim().length() != 0))
      comment = "# " + comment;
    comments.add(comment);
  }
  

  /**
   * Adds a comment to the tag while reading a file.
   *
   * @param   comment  The comment
   */
  protected void addCommentFromFile(String comment){
    mem.add(comment);
  }
  

  /**
   * The level just before the tag itself! ( For the "User" tag that will be the "Users" tag)<br>
   * (Syntax Users.User)
   *
   * @param   optionName  
   * @return     the XMLTag
   * @exception   NoSuchTagException  
   */
  private XMLTag findTagLevel(String tagName)throws NoSuchTagException{
    StringTokenizer st = new StringTokenizer(tagName,".");

    XMLTag cur = this;
    XMLTag tmp;
    String name="";
    int tokens = st.countTokens()-1;
    boolean found=false;
    for(int i = 0 ; i< tokens; i++){
      name = st.nextToken();
      for(int j = 0 ; j<cur.tags.size();j++){
        tmp = (XMLTag) cur.tags.get(j);
        if (name.equals(tmp.tagName)){
          cur = tmp;
          found=true;
          break;
        }else
          found=false;
      }
      if(!found)
        throw new NoSuchTagException("Tagget <"+name+"> findes ikke!");
    }
    return cur;
  }
  

  /**
   * Simply returns the value after the last dot in optionName. If there are no dots,
	 * the optionName itself is returned.
   *
   * Syntax (Users.User.name)
   *
   * @param   optionName  (Users.User.name)
   * @return     OptionName (name)
   */
  private String getOptionName(String optionName){
    if (optionName.lastIndexOf('.') != -1)
      return optionName.substring(optionName.lastIndexOf('.')+1, optionName.length());
    else
      return optionName;

  }


  /**
   *  Sets the value of an option in this tag.
   *
   * @param   optionName  the name of an option
   * @param   optionValue  The new value of this option
   * @exception   NoSuchOptionException  If the option are not found, an exception is thrown.
   */
  public void setOptionValue(String optionName, String optionValue) throws NoSuchOptionException{
    for (int i = 0 ; i < options.size() ; i++){
      String[] arr = (String[]) options.get(i);
      if (arr[0].equals(optionName)){
        arr[1]=optionValue;
        return;
      }
    }
		throw new NoSuchOptionException("Option ["+optionName+"] findes ikke i <"+tagName+">!");
  }
  
  /**
   * Sets the value of an option. The syntax for the method is (Users.User.name)
   *
   * @param   optionName  the options name (and XMLTags to it)
   * @param   optionValue  The new value of this option
   * @exception   NoSuchTagException  If the option are not found, an exception is thrown.
   */
  public void setQuickOptionValue(String optionName, String optionValue) throws NoSuchTagException{
    String name = getOptionName(optionName);
    XMLTag cur = findTagLevel(optionName);
    cur.setOptionValue(name,optionValue);
  }

  /**
   * Gets the value of an option in this tag.
   *
   * @param   optionName  The name of the option witch value is requested.
   * @return     value
   * @exception   NoSuchOptionException  If the option are not found, an exception is thrown.
   */
  public String getOptionValue(String optionName) throws NoSuchOptionException{
    String[] op;
    for(int i = 0 ; i < options.size(); i++){
      op = (String[]) options.get(i);
      if (op[0].equals(optionName))
        return op[1];
    }
      throw new NoSuchOptionException("Option ["+optionName+"] findes ikke i <"+tagName+">!");
  }
  
  /**
   * Quick way to find a value from tag-string of a certain syntax (Users.User.name)
   *
   * @param   optionName  The path from this tag, to the wished tag, along with the optionName (Users.User.name)
   * @return     value
   * @exception   NoSuchTagException  if one of the tags, or the option are not found, an exception is thrown
   */
  public String getQuickOptionValue(String optionName) throws NoSuchTagException {
    String name = getOptionName(optionName);
    XMLTag cur = findTagLevel(optionName);
    return cur.getOptionValue(name);
  }

  /**
	 * Finds all tags, that has the passed Name,
   * Syntax (Users.User).
   * 
   * @param   tagName  the tags name
   * @return     All qualyfied tags in an array[].
   * @exception   NoSuchTagException  If no tags are found with the passed name.
   */
  public XMLTag[] getTagsLike(String tagName) throws NoSuchTagException{
    Vector v = new Vector();
    
    String name = getOptionName(tagName);
    XMLTag cur = findTagLevel(tagName);
    XMLTag tmp;
    for(int i = 0 ; i < cur.tags.size(); i++){
      tmp = (XMLTag) cur.tags.get(i);
      if (tmp.tagName.equals(name))
        v.add(tmp);
    }
    
    if (v.size()==0)
      throw new NoSuchTagException("Tagget <"+tagName+"> findes ikke!");

    XMLTag[] xml = new XMLTag[v.size()];
    for (int i = 0 ; i < xml.length ; i ++)
      xml[i] = (XMLTag) v.get(i);
    return xml;
  }
  

  /**
   * Finds the first tag, that has the passed name.
   * Syntax (Users.User).
   *
   * @param   tagName  the name of tag that is searched for
   * @return     the teg
   * @exception   NoSuchTagException  If the tag are not found.
   */
  public XMLTag getTagLike(String tagName) throws NoSuchTagException{
    XMLTag[] t = getTagsLike(tagName);
    return t[0];
  }
	

	/**
	 * Returns all the subTags of this class.
	 *
	 * @return     an empty array if no tags are found.
	 */
	public XMLTag[] getSubTags() {
		XMLTag[] arr = new XMLTag[tags.size()];
		
		for (int i = 0 ; i < arr.length ; i++)
			arr[i] = (XMLTag) tags.get(i);
		return arr;	
	}
  

  /**
   * Henter hele XML strukturen fra dette tag og nedefter (i sub-Tags). 
   * Strukturen retuneres pnt opsat, og klar til at blive skrevet i en fil.
   *
   * @return     XML dokument.
   */
  public String getXMLString(){
    return getXMLString(new StringBuffer(3000), 0);
  }
  

  /**
	 * The method is recursiv, and is used by getXMLString() to build a XML-document.
   * Metoden er rekursiv, og bruges af getXMLString() til at opbygge XML-dokument.
   *
   * @param   xml  The XML-document
   * @param   space  Number of spaces before options and tags. (is used to get a nice setup)
   * @return     The XML-document
   */
  private String getXMLString(StringBuffer xml, int space){
    String tagName;

    XMLTag t;
    for(int i = 0 ; i < tags.size(); i++){
      t = (XMLTag)tags.get(i);
      
      for(int j = 0 ; j < t.comments.size(); j++){
        xml.append(t.comments.get(j).toString() + "\n");
      }
      
      xml.append( makeString("<" + t.tagName + ">\n", space));
      
      String[] op;
      for(int j = 0 ; j < t.options.size(); j++){
        op = (String[]) t.options.get(j);
        xml.append(makeString(op[0] + " ",space+2));
        xml.append(op[1] + "\n");
      }
      t.getXMLString(xml,space+2);
            
      xml.append( makeString("</" + t.tagName + ">\n",space));
    }

    return xml.toString();
  }
  

  /**
   * Sets a number of spaces spaces before "st".
   *
   * @param   st  This string will be returned with the spaces infront.
   * @param   space  number of spaces.
   * @return     " " * spaces + st (a matter of speaking)
   */
  private String makeString(String st, int space){
    String tmp="";
    for(int i = 0 ; i < space; i++){
      tmp+=" ";
    }
    return tmp + st;
  }
  
  

  /**
   * Adds a sub-Tag to this tag. <br>
   * &ltTAG&gt 
   * &nbsp;&nbsp;&ltSUBTAG&gt
   * &nbsp;&nbsp;&lt/SUBTAG&gt
   * &lt/TAG&gt 
   * @param   tagName  The name of the sub-Tag
   * @return     nyt Tag
   */
  protected XMLTag addTagFromFile(String tagName){
    if (okToAdd){
      okToAdd=false; //tag must be ended before next tag!
      
      XMLTag t =new XMLTag(tagName);
			t.setParent(this);
      t.comments = mem;
      mem = new Vector();  

      tags.add(t);
      return t;
    }else{
      return ((XMLTag) tags.lastElement()).addTagFromFile(tagName);
    }
  }
  

	/**
	 * Sets the parrent Tag. The methos is automatically called, when the tag is added to another tag.
	 *
	 * @param   tag  parrent tag
	 */
  private void setParent(XMLTag tag) {
  	parent=tag;
  }
  
	/**
	 * The tag, that this tag has been addd to.
	 *
	 * @return     null, if there are no parent tag.
	 */
	public XMLTag getParent() {
		return parent;
	}

  /**
   * Adds a new tag to this tag.
   *
   * @param   newTag  the new tag.
   */
  public void addTag(XMLTag newTag){
    if (!tags.contains(newTag)) {
			tags.add(newTag);
			newTag.setParent(this);
    }		
  }
	
	/**
	 * Removes a tag from this tag.
	 *
	 * @param   oldTag  the tag that is to be removed
	 */
	public boolean removeTag(XMLTag oldTag) {
		return tags.remove(oldTag);
	}

  /**
   * Ends a tag, that has been readen from a file. States that a following tag may be inserted.
   *
   * @param   tagName  The name of the tag, that shall be ended.
   * @return     false, if the tag cannot be ended!
   */
  protected boolean endTag(String tagName){
    
    if (tags.size() <= 0)
      return false;
      
    XMLTag tag = (XMLTag) tags.lastElement();
    if (tag!=null){
      if (tag.okToAdd){
        if (tag.tagName.equals(tagName)){
          okToAdd = true;
          return okToAdd;
        }else{
          return false;
        }
      }else
        return tag.endTag(tagName);
    }
    return false;
  }
  

	/**
	 * Prints the XMLTag, and all its underlaying tags as a String.
	 *
	 * @return     tag structure.
	 */
/*  public String toString(){
    return getXMLString();
  }*/
}